from flask_login import current_user
from flask_socketio import join_room, leave_room
from app import hashids, socketio
from app.decorators import socketio_login_required
from app.models import User


@socketio.on('SUBSCRIBE /users/<user_id>')
@socketio_login_required
def subscribe_user(user_hashid):
    user_id = hashids.decode(user_hashid)
    user = User.query.get(user_id)
    if user is None:
        return {'code': 404, 'msg': 'Not found'}
    if not (user == current_user or current_user.is_administrator):
        return {'code': 403, 'msg': 'Forbidden'}
    join_room(f'/users/{user.hashid}')
    return {'code': 200, 'msg': 'OK'}


@socketio.on('UNSUBSCRIBE /users/<user_id>')
@socketio_login_required
def unsubscribe_user(user_hashid):
    user_id = hashids.decode(user_hashid)
    user = User.query.get(user_id)
    if user is None:
        return {'code': 404, 'msg': 'Not found'}
    if not (user == current_user or current_user.is_administrator):
        return {'code': 403, 'msg': 'Forbidden'}
    leave_room(f'/users/{user.hashid}')
    return {'code': 200, 'msg': 'OK'}
